"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const fordpass_1 = require("./fordpass");
const vehicle_1 = require("./types/vehicle");
const fordpass_connection_1 = require("./fordpass-connection");
const accessory_1 = require("./accessory");
let hap;
let Accessory;
const PLUGIN_NAME = 'homebridge-fordpass';
const PLATFORM_NAME = 'FordPass';
class FordPassPlatform {
    constructor(log, config, api) {
        this.accessories = [];
        this.vehicles = [];
        this.pendingLockUpdate = false;
        this.log = log;
        this.api = api;
        this.config = config;
        if (!config) {
            return;
        }
        if (!config.username || !config.password) {
            this.log.error('Please add a userame and password to your config.json');
            return;
        }
        api.on("didFinishLaunching", this.didFinishLaunching.bind(this));
    }
    configureAccessory(accessory) {
        var _a, _b, _c;
        const self = this;
        this.log.info(`Configuring accessory ${accessory.displayName}`);
        accessory.on("identify", () => {
            this.log.info(`${accessory.displayName} identified!`);
        });
        const vehicle = new fordpass_1.Vehicle(accessory.context.name, accessory.context.vin, this.config, this.log);
        const fordAccessory = new accessory_1.FordpassAccessory(accessory);
        const defaultState = hap.Characteristic.LockTargetState.UNSECURED;
        const lockService = fordAccessory.createService(hap.Service.LockMechanism);
        const switchService = fordAccessory.createService(hap.Service.Switch);
        const batteryService = fordAccessory.createService(hap.Service.Battery, ((_a = this.config.options) === null || _a === void 0 ? void 0 : _a.batteryName) || 'Fuel Level');
        if ((_b = this.config.options) === null || _b === void 0 ? void 0 : _b.chargingSwitch) {
            fordAccessory.createService(hap.Service.OccupancySensor, 'Charging');
        }
        else {
            fordAccessory.removeService(hap.Service.OccupancySensor, 'Charging');
        }
        if ((_c = this.config.options) === null || _c === void 0 ? void 0 : _c.plugSwitch) {
            fordAccessory.createService(hap.Service.OccupancySensor, 'Plug');
        }
        else {
            fordAccessory.removeService(hap.Service.OccupancySensor, 'Plug');
        }
        lockService.setCharacteristic(hap.Characteristic.LockCurrentState, defaultState);
        lockService
            .setCharacteristic(hap.Characteristic.LockTargetState, defaultState)
            .getCharacteristic(hap.Characteristic.LockTargetState)
            .on("set", (value, callback) => __awaiter(this, void 0, void 0, function* () {
            this.log.debug(`${value ? 'Locking' : 'Unlocking'} ${accessory.displayName}`);
            let commandId = '';
            let command = vehicle_1.Command.LOCK;
            if (value === hap.Characteristic.LockTargetState.UNSECURED) {
                command = vehicle_1.Command.UNLOCK;
            }
            commandId = yield vehicle.issueCommand(command);
            let tries = 30;
            this.pendingLockUpdate = true;
            const self = this;
            const interval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
                if (tries > 0) {
                    const status = yield vehicle.commandStatus(commandId);
                    if ((status === null || status === void 0 ? void 0 : status.currentStatus) === 'SUCCESS') {
                        lockService.updateCharacteristic(hap.Characteristic.LockCurrentState, value);
                        self.pendingLockUpdate = false;
                        clearInterval(interval);
                    }
                    tries--;
                }
                else {
                    self.pendingLockUpdate = false;
                    clearInterval(interval);
                }
            }), 3000);
            callback(undefined, value);
        }))
            .on("get", (callback) => __awaiter(this, void 0, void 0, function* () {
            var _d, _e, _f;
            let lockNumber = hap.Characteristic.LockTargetState.UNSECURED;
            const lockStatus = ((_e = (_d = vehicle === null || vehicle === void 0 ? void 0 : vehicle.info) === null || _d === void 0 ? void 0 : _d.metrics.doorLockStatus.find((x) => x.vehicleDoor === 'ALL_DOORS')) === null || _e === void 0 ? void 0 : _e.value) || 'LOCKED';
            if (lockStatus === 'LOCKED') {
                lockNumber = hap.Characteristic.LockTargetState.SECURED;
            }
            callback(undefined, lockNumber);
            if (!this.config.access_token || !vehicle) {
                return;
            }
            const status = yield vehicle.status();
            if (status) {
                let lockNumber = hap.Characteristic.LockTargetState.UNSECURED;
                const lockStatus = (_f = status.metrics.doorLockStatus.find((x) => x.vehicleDoor === 'ALL_DOORS')) === null || _f === void 0 ? void 0 : _f.value;
                if (lockStatus === 'LOCKED') {
                    lockNumber = hap.Characteristic.LockTargetState.SECURED;
                }
                lockService.updateCharacteristic(hap.Characteristic.LockCurrentState, lockNumber);
                lockService.updateCharacteristic(hap.Characteristic.LockTargetState, lockNumber);
            }
            else {
                self.log.error(`Cannot get information for ${accessory.displayName} lock`);
            }
        }));
        switchService
            .setCharacteristic(hap.Characteristic.On, false)
            .getCharacteristic(hap.Characteristic.On)
            .on("set", (value, callback) => __awaiter(this, void 0, void 0, function* () {
            this.log.debug(`${value ? 'Starting' : 'Stopping'} ${accessory.displayName}`);
            if (value) {
                yield vehicle.issueCommand(vehicle_1.Command.START);
            }
            else {
                yield vehicle.issueCommand(vehicle_1.Command.STOP);
            }
            callback(undefined, value);
        }))
            .on("get", (callback) => __awaiter(this, void 0, void 0, function* () {
            var _g;
            const engineStatus = ((_g = vehicle === null || vehicle === void 0 ? void 0 : vehicle.info) === null || _g === void 0 ? void 0 : _g.metrics.ignitionStatus.value) || 'OFF';
            callback(undefined, engineStatus);
            if (!this.config.access_token) {
                return;
            }
            const status = yield vehicle.status();
            if (status) {
                let started = true;
                const engineStatus = status.metrics.ignitionStatus.value;
                if (engineStatus === 'OFF') {
                    started = false;
                }
                switchService.updateCharacteristic(hap.Characteristic.On, started);
            }
            else {
                self.log.error(`Cannot get information for ${accessory.displayName} engine`);
            }
        }));
        batteryService
            .setCharacteristic(hap.Characteristic.BatteryLevel, 100)
            .getCharacteristic(hap.Characteristic.BatteryLevel)
            .on("get", (callback) => __awaiter(this, void 0, void 0, function* () {
            var _h, _j, _k, _l, _m, _o, _p, _q;
            const fuel = (_j = (_h = vehicle === null || vehicle === void 0 ? void 0 : vehicle.info) === null || _h === void 0 ? void 0 : _h.metrics.fuelLevel) === null || _j === void 0 ? void 0 : _j.value;
            const battery = (_l = (_k = vehicle === null || vehicle === void 0 ? void 0 : vehicle.info) === null || _k === void 0 ? void 0 : _k.metrics.xevBatteryStateOfCharge) === null || _l === void 0 ? void 0 : _l.value;
            let level = fuel || battery || 100;
            if (level > 100) {
                level = 100;
            }
            if (level < 0) {
                level = 0;
            }
            callback(undefined, level);
            if (!this.config.access_token) {
                return;
            }
            const status = yield vehicle.status();
            if (status) {
                const fuel = (_m = status.metrics.fuelLevel) === null || _m === void 0 ? void 0 : _m.value;
                const battery = (_o = status.metrics.xevBatteryStateOfCharge) === null || _o === void 0 ? void 0 : _o.value;
                const chargingStatus = (_q = (_p = vehicle === null || vehicle === void 0 ? void 0 : vehicle.info) === null || _p === void 0 ? void 0 : _p.metrics.xevBatteryChargeDisplayStatus) === null || _q === void 0 ? void 0 : _q.value;
                let level = fuel || battery || 100;
                if (level > 100) {
                    level = 100;
                }
                if (level < 0) {
                    level = 0;
                }
                batteryService.updateCharacteristic(hap.Characteristic.BatteryLevel, level);
                if (battery) {
                    if (chargingStatus === 'ChargingAC') {
                        batteryService.updateCharacteristic(hap.Characteristic.ChargingState, hap.Characteristic.ChargingState.CHARGING);
                    }
                    else {
                        batteryService.updateCharacteristic(hap.Characteristic.ChargingState, hap.Characteristic.ChargingState.NOT_CHARGING);
                    }
                }
                else {
                    batteryService.updateCharacteristic(hap.Characteristic.ChargingState, hap.Characteristic.ChargingState.NOT_CHARGEABLE);
                }
                if (level < 10) {
                    batteryService.updateCharacteristic(hap.Characteristic.StatusLowBattery, hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW);
                }
                else {
                    batteryService.updateCharacteristic(hap.Characteristic.StatusLowBattery, hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL);
                }
            }
            else {
                self.log.error(`Cannot get information for ${accessory.displayName} engine`);
            }
        }));
        this.vehicles.push(vehicle);
        this.accessories.push(accessory);
    }
    didFinishLaunching() {
        return __awaiter(this, void 0, void 0, function* () {
            const self = this;
            const ford = new fordpass_connection_1.Connection(this.config, this.log);
            const authInfo = yield ford.auth();
            if (authInfo) {
                setInterval(() => __awaiter(this, void 0, void 0, function* () {
                    self.log.debug('Reauthenticating with refresh token');
                    yield ford.refreshAuth();
                }), authInfo.expires_in * 1000 - 10000);
                yield this.addVehicles(ford);
                yield this.updateVehicles();
                yield this.refreshVehicles();
                setInterval(() => __awaiter(this, void 0, void 0, function* () {
                    yield self.updateVehicles();
                }), 60 * 1000 * 15);
            }
        });
    }
    addVehicles(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const vehicles = yield connection.getVehicles();
            vehicles === null || vehicles === void 0 ? void 0 : vehicles.forEach((vehicle) => __awaiter(this, void 0, void 0, function* () {
                vehicle.VIN = vehicle.VIN.toUpperCase();
                const name = vehicle.nickName || vehicle.year + ' ' + vehicle.make + ' ' + vehicle.model;
                const uuid = hap.uuid.generate(vehicle.VIN);
                const accessory = new Accessory(name, uuid);
                accessory.context.name = name;
                accessory.context.vin = vehicle.VIN;
                const accessoryInformation = accessory.getService(hap.Service.AccessoryInformation);
                if (accessoryInformation) {
                    accessoryInformation.setCharacteristic(hap.Characteristic.Manufacturer, 'Ford');
                    accessoryInformation.setCharacteristic(hap.Characteristic.Model, name);
                    accessoryInformation.setCharacteristic(hap.Characteristic.SerialNumber, vehicle.VIN);
                }
                if (!this.accessories.find((x) => x.UUID === uuid)) {
                    this.log.debug(`New vehicle found: ${name}`);
                    this.configureAccessory(accessory);
                    this.api.registerPlatformAccessories(PLUGIN_NAME, PLATFORM_NAME, [accessory]);
                }
            }));
            this.accessories.forEach((accessory) => {
                if (!(vehicles === null || vehicles === void 0 ? void 0 : vehicles.find((x) => x.VIN === accessory.context.vin))) {
                    this.api.unregisterPlatformAccessories(PLUGIN_NAME, PLATFORM_NAME, [accessory]);
                    const index = this.accessories.indexOf(accessory);
                    if (index > -1) {
                        this.accessories.splice(index, 1);
                        this.vehicles.slice(index, 1);
                    }
                }
            });
        });
    }
    updateVehicles() {
        return __awaiter(this, void 0, void 0, function* () {
            this.vehicles.forEach((vehicle) => __awaiter(this, void 0, void 0, function* () {
                var _a, _b, _c;
                const status = yield vehicle.status();
                if (!status) {
                    return;
                }
                this.log.debug(`Updating info for ${vehicle.name}`);
                this.log.debug(`Vehicle Status : ${JSON.stringify(status, null, 2)}`);
                const lockStatus = (_a = status.metrics.doorLockStatus.find((x) => x.vehicleDoor === 'ALL_DOORS')) === null || _a === void 0 ? void 0 : _a.value;
                let lockNumber = hap.Characteristic.LockCurrentState.UNSECURED;
                if (lockStatus === 'LOCKED') {
                    lockNumber = hap.Characteristic.LockCurrentState.SECURED;
                }
                const engineStatus = status.metrics.ignitionStatus.value;
                let started = true;
                if (engineStatus === 'OFF') {
                    started = false;
                }
                const uuid = hap.uuid.generate(vehicle.vin);
                const accessory = this.accessories.find((x) => x.UUID === uuid);
                if (accessory) {
                    const fordAccessory = new accessory_1.FordpassAccessory(accessory);
                    if (!this.pendingLockUpdate) {
                        const lockService = fordAccessory.findService(hap.Service.LockMechanism);
                        lockService && lockService.updateCharacteristic(hap.Characteristic.LockCurrentState, lockNumber);
                        lockService && lockService.updateCharacteristic(hap.Characteristic.LockTargetState, lockNumber);
                    }
                    const switchService = fordAccessory.findService(hap.Service.Switch);
                    switchService && switchService.updateCharacteristic(hap.Characteristic.On, started);
                    const plugService = fordAccessory.findService(hap.Service.OccupancySensor, 'Plug');
                    plugService &&
                        plugService.updateCharacteristic(hap.Characteristic.OccupancyDetected, ((_b = status === null || status === void 0 ? void 0 : status.metrics.xevPlugChargerStatus) === null || _b === void 0 ? void 0 : _b.value)
                            ? hap.Characteristic.OccupancyDetected.OCCUPANCY_DETECTED
                            : hap.Characteristic.OccupancyDetected.OCCUPANCY_NOT_DETECTED);
                    const chargingService = fordAccessory.findService(hap.Service.OccupancySensor, 'Charging');
                    chargingService &&
                        chargingService.updateCharacteristic(hap.Characteristic.OccupancyDetected, ((_c = status === null || status === void 0 ? void 0 : status.metrics.xevBatteryChargeDisplayStatus) === null || _c === void 0 ? void 0 : _c.value) === 'ChargingAC'
                            ? hap.Characteristic.OccupancyDetected.OCCUPANCY_DETECTED
                            : hap.Characteristic.OccupancyDetected.OCCUPANCY_NOT_DETECTED);
                }
                else {
                    this.log.warn(`Accessory not found for ${vehicle.name}`);
                }
            }));
        });
    }
    refreshVehicles() {
        return __awaiter(this, void 0, void 0, function* () {
            this.vehicles.forEach((vehicle) => __awaiter(this, void 0, void 0, function* () {
                if (vehicle.autoRefresh && vehicle.refreshRate && vehicle.refreshRate > 0) {
                    this.log.debug(`Configuring ${vehicle.name} to refresh every ${vehicle.refreshRate} minutes.`);
                    setInterval(() => __awaiter(this, void 0, void 0, function* () {
                        this.log.debug(`Refreshing info for ${vehicle.name}`);
                        yield vehicle.issueCommand(vehicle_1.Command.REFRESH);
                    }), 60000 * vehicle.refreshRate);
                }
            }));
        });
    }
}
module.exports = (api) => {
    hap = api.hap;
    Accessory = api.platformAccessory;
    api.registerPlatform(PLUGIN_NAME, PLATFORM_NAME, FordPassPlatform);
};
//# sourceMappingURL=index.js.map